
///////////////////////////////////////////////////////////////////////////
// This program is free software: you can redistribute it and/or modify  //
// it under the terms of the version 3 of the GNU General Public License //
// as published by the Free Software Foundation.                         //
//                                                                       //
// This program is distributed in the hope that it will be useful, but   //
// WITHOUT ANY WARRANTY; without even the implied warranty of            //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      //
// General Public License for more details.                              //
//                                                                       //
// You should have received a copy of the GNU General Public License     //
// along with this program. If not, see <http://www.gnu.org/licenses/>.  //
//                                                                       //
// Written and (C) by Francois Fleuret                                   //
// Contact <francois.fleuret@idiap.ch> for comments & bug reports        //
///////////////////////////////////////////////////////////////////////////

#ifndef MAPPINGS_H
#define MAPPINGS_H

#include <iostream>

using namespace std;

// We have a bunch of such classes inside
class Map;

class Mapping {
  Map *f;

  // We'll need this one from time to time, but nobody is supposed to
  // use it from "outside"
  Mapping(Map *g);

public:
  // This is the variable
  const static Mapping X;

  // This builds a non-defined Mapping. Everything is illegal on such
  // a Mapping except delete, = and <<. All other operations will
  // *crash*
  Mapping();

  // Cast from float to allow operations with constant values
  Mapping(float x);

  // Standard copy constructor and destructor
  Mapping(const Mapping &s);
  ~Mapping();

  // Assignment
  Mapping &operator = (const Mapping &m);

  // Evaluation
  float operator () (float x) const;

  // Derivation
  Mapping derivative() const;

  // Composition of Mappings
  Mapping compose(const Mapping &m) const;

  // Standard operations
  Mapping pow(int k) const;
  Mapping mul(const Mapping &m) const;
  Mapping div(const Mapping &m) const;
  Mapping add(const Mapping &m) const;
  Mapping sub(const Mapping &m) const;
  Mapping neg() const;
  Mapping sin() const;
  Mapping cos() const;
  Mapping log() const;
  Mapping exp() const;

  // Print the expression
  void print(ostream &s) const;
};

// I guess we have all the ones we need?
Mapping operator + (const Mapping &m);
Mapping operator + (const Mapping &ml, const Mapping &mr);
Mapping operator - (const Mapping &m);
Mapping operator - (const Mapping &ml, const Mapping &mr);
Mapping operator * (const Mapping &ml, const Mapping &mr);
Mapping operator / (const Mapping &ml, const Mapping &mr);

// Be very careful with this operator, it has a lower precedence than
// +-*/
Mapping operator ^ (const Mapping &ml, int k);

// stream stuff, I dont have the courage to do the >>
ostream &operator << (ostream &o, const Mapping &m);

Mapping sin(const Mapping &m);
Mapping cos(const Mapping &m);
Mapping log(const Mapping &m);
Mapping exp(const Mapping &m);

#endif
